"""
Copyright 2024 Odd Gunnar Aspaas

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import requests
from requests import Response
from .utils.note import Note
from typing import Optional


class Client:
    """
    A client for interacting with the Slite API.
    """

    def __init__(self, api_key: str):
        """
        Initializes the Slite API client.

        Args:
            api_key: Your Slite API key.
        """
        if not api_key:
            raise ValueError("API key cannot be empty.")
        self.api_key = api_key
        self.headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "x-slite-api-key": self.api_key,
        }
        self.base_url = "https://api.slite.com/v1/"

    def get_note(self, note_id: str) -> Optional[Note]:
        """
        Retrieves a note from Slite by its ID.

        Args:
            note_id: The ID of the note to retrieve.

        Returns:
            A Note object if the note is found, None otherwise.

        Raises:
            requests.exceptions.RequestException: If there's an error during the HTTP request.
        """
        url = self.base_url + f"notes/{note_id}?format=md"
        try:
            response = requests.get(url, headers=self.headers)
            response.raise_for_status()  # Raise an exception for bad status codes
            return Note(**response.json())
        except requests.exceptions.RequestException as e:
            print(f"Error fetching note: {e}")
            return None
        except ValueError as e:
            print(f"Error parsing JSON response when fetching note: {e}")
            return None

    def create_note(
        self, parent_note_id: str, template_id: str, title: str, content: str
    ) -> Optional[Response]:
        """
        Creates a new note in Slite.

        Args:
            parent_note_id: The ID of the parent note.
            template_id: The ID of the template to use.
            title: The title of the new note.
            content: The content of the new note.

        Returns:
            The response object from the Slite API, or None if failed.
        Raises:
            requests.exceptions.RequestException: If there's an error during the HTTP request.
        """

        url = self.base_url + "notes/"
        payload = {
            "title": title,
            "parentNoteId": parent_note_id,
            "templateId": template_id,
            "markdown": content,
        }
        try:
            response = requests.post(url, json=payload, headers=self.headers)
            response.raise_for_status()
            return response

        except requests.exceptions.RequestException as e:
            print(f"Error creating note: {e}")
            return None
        except ValueError as e:
            print(f"Error parsing JSON response when creating note: {e}")
            return None



import dataclasses
from datetime import datetime
import json
from dataclasses import dataclass, field
from typing import List

from typing import Optional

from enum import Enum

class ReviewState(Enum):
  Verified = "Verified"
  Outdated = "Outdated"
  VerificationRequested = "VerificationRequested"

@dataclass
class Note:
  updatedAt: datetime
  columns: Optional[List]
  attributes: Optional[List]
  url: str
  parentNoteId: str
  title: str
  id: str
  content: str
  reviewState: Optional[ReviewState] = None


  def __post_init__(self):
    if self.columns and self.attributes:
      print("ITS A COLLECTION")
      self.collection = True


  def show(self):
    try:
      from IPython import display, get_ipython
      get_ipython()
      if self.content:
        content = f"#{self.title}\n" + self.content
        display.display(display.Markdown(content))
      else:
        return "No Content"
    except:
      print(self.title)
      print(self.content)


  def json(self):
    clean_dict = {}
    for key,value in dataclasses.asdict(self).items():
      if value:
        clean_dict[key] = value
    return json.dumps(dataclasses.asdict(self))


import json
from dataclasses import dataclass, asdict
from typing import Optional


@dataclass
class Tile:
  title: str = ""
  iconURL: str = None
  colorHex: str = None
  label: str = None
  url: str = None
  content: str = None

  def __post_init__(self):
    if self.colorHex and not self.label:
      raise ValueError("Can not have status without label.")


  def json(self):
    clean_dict = {}
    status = {}
    for key,value in asdict(self).items():
      if value:
        if key in ["label", "colorHex"]:
          status[key] = value
        else:
          clean_dict[key] = value
    if status:
      clean_dict["status"] = status
    return json.dumps(asdict(self))